@@ -895,13 +895,13 @@ virSysinfoParseX86Chassis(const char *base,
 
 
 static int
-virSysinfoDMIDecodeOEMString(size_t i,
+virSysinfoDMIDecodeOEMString(unsigned int idx,
                              char **str)
 {
     g_autofree char *err = NULL;
     g_autoptr(virCommand) cmd = virCommandNewArgList(DMIDECODE, "--dump",
                                                      "--oem-string", NULL);
-    virCommandAddArgFormat(cmd, "%zu", i);
+    virCommandAddArgFormat(cmd, "%u", idx);
     virCommandSetOutputBuffer(cmd, str);
     virCommandSetErrorBuffer(cmd, &err);
 
@@ -910,11 +910,23 @@ virSysinfoDMIDecodeOEMString(size_t i,
 
     /* Unfortunately, dmidecode returns 0 even if OEM String index is out
      * of bounds, but it prints an error message in that case. Check stderr
-     * and return success/failure accordingly. */
-
-    if (err && *err != '\0')
+     * and return success/failure accordingly.
+     * To make matters worse, if there are two or more 'OEM String'
+     * sections then:
+     *
+     * a) we have no way of distinguishing them as dmidecode prints
+     *    strings from all sections,
+     * b) if one section contains a valid string, but the other doesn't,
+     *    then stdout contains the valid string and stderr contains the
+     *    error "No OEM string number X*.
+     *
+     * Let's just hope there is not many systems like that.
+     */
+    if ((!*str || **str == '\0') && err && *err != '\0')
         return -1;
 
+    virStringTrimOptionalNewline(*str);
+
     return 0;
 }
 
@@ -924,7 +936,6 @@ virSysinfoParseOEMStrings(const char *base,
                           virSysinfoOEMStringsDef **stringsRet)
 {
     virSysinfoOEMStringsDef *strings = NULL;
-    size_t i = 1;
     int ret = -1;
     const char *cur;
 
@@ -934,6 +945,8 @@ virSysinfoParseOEMStrings(const char *base,
     strings = g_new0(virSysinfoOEMStringsDef, 1);
 
     while ((cur = strstr(cur, "String "))) {
+        char *collon = NULL;
+        unsigned int idx = 0;
         char *eol;
 
         cur += 7;
@@ -944,8 +957,13 @@ virSysinfoParseOEMStrings(const char *base,
             goto cleanup;
         }
 
-        while (g_ascii_isdigit(*cur))
-            cur++;
+        if (virStrToLong_ui(cur, &collon, 10, &idx) < 0) {
+            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                           _("Malformed output of dmidecode"));
+            goto cleanup;
+        }
+
+        cur = collon;
 
         if (*cur != ':') {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
@@ -968,7 +986,7 @@ virSysinfoParseOEMStrings(const char *base,
         if (memchr(cur, '.', eol - cur)) {
             char *str;
 
-            if (virSysinfoDMIDecodeOEMString(i, &str) < 0)
+            if (virSysinfoDMIDecodeOEMString(idx, &str) < 0)
                 goto cleanup;
 
             strings->values[strings->nvalues - 1] = g_steal_pointer(&str);
@@ -976,7 +994,6 @@ virSysinfoParseOEMStrings(const char *base,
             strings->values[strings->nvalues - 1] = g_strndup(cur, eol - cur);
         }
 
-        i++;
         cur = eol;
     }
 
@@ -1249,6 +1266,8 @@ virSysinfoRead(void)
     (defined(__x86_64__) || \
      defined(__i386__) || \
      defined(__amd64__) || \
+     defined(__riscv__) || \
+     defined(__mips__) || \
      defined(__loongarch__))
     return virSysinfoReadDMI();
 #else /* WIN32 || not supported arch */
